/*! jQuery-widget v1.0.0 License: https://github.com/smartystreets/jquery-widget/ */
(function($, window) {
    "use strict";

    $.fn.inject = function(options) {

        var settings = $.extend({
            timeout: 4000,
            attempts: 3, // 1-based value
            async: true,
            cache: false,
            dataType: 'text',
            css: {
                loading: 'widget-loading',
                success: 'widget-loaded',
                failure: 'widget-failed'
            },
            progress: noop,
            receive: receive,
            append: append,
            success: noop,
            failure: noop,
            complete: noop,
            exception: noop
        }, options || {});

        settings.base = parseUri(settings.base);

        return this.each(function() {
            var $this = $(this);

            ajax.call({
                context: $this,
                url: defineUrl.call($this, settings),
                settings: settings,
                attempt: 0
            });
        });
    };
    function noop() {
    }
    function defineUrl(settings) {
        var _this = this[0];
        var url = parseUri(this.attr('src') || settings.url);
        if (_this._base || !url)
            return undefined;

        if (relativeUrl(url))
            return undefined; // relative urls not supported.

        var base = parseUri(this.attr('base')) || settings.base || parseUri(url.prefix() + '/');
        if (relativeUrl(base))
            return undefined;

        if (!base.relative)
            base.relative = '/';

        _this._base = base.toString(); // use jQuery.data?

        if (!base.authority && url.authority)
            return undefined;

        if (settings.base && settings.base.authority && url.authority)
            return undefined;

        if (url.authority)
            return url.toString();

        return combineUrls(base, url);
    }
    function relativeUrl(url) {
        return url && !url.authority && url.path.indexOf('/') !== 0;
    }
    function combineUrls(first, second) {
        first = (first || '').toString();
        second = (second || '').toString();
        if (first && second && first.charAt(first.length - 1) === '/' && second.charAt(0) === '/')
            return first + second.substring(1);

        return first + second;
    }

    function ajax() {
        if (!this.url)
            return false;

        if (++this.attempt > this.settings.attempts)
            return false;

        this.context.addClass(this.settings.css.loading);
        this.settings.progress.call(this.context, this.attempt);
        var _this = this;

        return $.ajax({
            url: _this.url,
            type: 'get',
            context: _this.context,
            cache: _this.settings.cache,
            async: _this.settings.async,
            timeout: _this.settings.timeout,
            dataType: _this.settings.dataType,
            complete: function(xhr) { tryAjaxComplete.call(_this, xhr); }
        }) || true;
    }
    function tryAjaxComplete(xhr) {
        this.context.removeClass(this.settings.css.loading);

        try {
            if (!ajaxComplete.call(this, xhr))
                return;

            this.context.addClass(this.settings.css.success);
            this.settings.complete.call(this.context, xhr);
        } catch (e) {
            var console = window.console;
            if (console && console.log)
                console.log(e);

            this.context.addClass(this.settings.css.failure);
            this.settings.exception.call(this.context, e);
            this.settings.complete.call(this.context, xhr);
        }
    }
    function ajaxComplete(xhr) {
        if (xhr.status === 200)
            rebase.call(this, xhr);
        else if (xhr.status === 404)
            failure.call(this, xhr);
        else if ((xhr.status === 500 || xhr.statusText === 'timeout') && !ajax.call(this))
            failure.call(this, xhr);
        else if (xhr.status === 500 || xhr.statusText === 'timeout')
            return false;

        return true;
    }
    function rebase(xhr) {
        var content = this.settings.receive.call(this.context, xhr, xhr.responseText || '');
        content = transform.call(this.context, content  || '');
        this.settings.append.call(this.context, content || '', this.settings);
        this.settings.success.call(this.context, xhr);
    }
    function receive(_, raw) {
        return raw;
    }
    function transform(raw) {
        var base = this[0]._base;

        return raw
            .replace(htmlEmbedded, function() { return transformElement.apply(base, arguments); })
            .replace(jsEmbedded, function() { return transformElement.apply(base, arguments); })
            .replace(widgetBase, base);
    }
    function transformElement(matched, pre, url, post) {
        if (url.indexOf('/') !== 0)
            return matched; // relative url, ignore because we don't deal with relative urls yet

        if (url.indexOf(widgetBase) !== -1 || fullyQualified.test(url) || rebaseNone.test(matched))
            return matched;

        return pre + combineUrls(this, url) + post;
    }
    function append(raw, _) {
        this.append(raw);

    }
    function failure(xhr) {
        this.context.addClass(this.settings.css.failure);
        this.settings.failure.call(this.context, xhr);
    }

    // parseUri 1.2.2 / (c) Steven Levithan <stevenlevithan.com> (additions by Jonathan Oliver) / MIT License
    function parseUri(str) {
        if (!str) return undefined;
        str = formatUri(str);

        var o = parseUri.options,
            m = o.parser.exec(str),
            uri = {
                toString: function() { return this.prefix() + this[o.key[8]]; },
                prefix: function() {
                    var p = this[o.key[1]];
                    var a = this[o.key[2]];
                    return (p?p+':':'')+(a?'//'+a:'');
                }
            },
            i = 14;

        while (i--) uri[o.key[i]] = m[i] || '';
        for (i = 1; i < 3; i++)
            uri[o.key[i]] = uri[o.key[i]].toLowerCase();

        uri[o.q.name] = {};
        uri[o.key[12]].replace(o.q.parser, function($0, $1, $2) {
            if ($1) uri[o.q.name][$1] = $2;
        });

        return uri;
    }
    function formatUri(url) {
        url = (url || '').toString();
        var hostname = (window.widget_location || window.location).hostname || '';
        var matches = hostname.match(tld);
        return url.replace('{0}', matches ? matches[0].replace('.', '') : 'com');
    }

    parseUri.options = {
        key: ["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","anchor"],
        q: {
            name: "queryKey",
            parser: /(?:^|&)([^&=]*)=?([^&]*)/g
        },
        parser: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/
    };

    var htmlEmbedded = /(<\s*?\w+[^>]*(?:src|href|action)\s*?=\s*?(?:'|")?)([^'"> ]+)((?:'|")?[^>]*\s?>)/ig; // TODO: don't match urls that contain authority component
    var jsEmbedded = /(url\s*?:\s*?(?:'|")?)([^'" ]+)((?:'|")?\s*?(?:,|;|\}))/ig;
    var widgetBase = 'widget://base/';
    var fullyQualified = /^(\w+:)?\/\/.*/ig; // starts with file://, http:// https://, etc. or just //
    var rebaseNone = /\srebase\s*=\s*('|")?none('|")?/ig;
    var tld = /\.([a-z0-9]+)$/ig; // TODO: prevent this from matching IPs
    window.document.createElement("widget"); // IE7-8 compatibility with custom 'widget' tag.

})(jQuery, window);